import 'dart:async';

import 'package:flutter/services.dart';

typedef void ApplePayCalback(
    Object pass, String errorCode, String errorMessage);

class FooApplePay {
  static ApplePayCalback applePayCallback;
  static const MethodChannel _methodChannel =
      const MethodChannel('foo_apple_pay');
  static const EventChannel _eventChannel =
      const EventChannel('foo_apple_pay_events');

  //Method Channel
  static void setHostNameAndPath(String hostName, String path) {
    _methodChannel.invokeMethod('setHostNameAndPath', [hostName, path]);
  }

  static Future<bool> deviceSupportsAppleWallet() async {
    return await _methodChannel.invokeMethod<bool>('deviceSupportsAppleWallet');
  }

  static Future<List<dynamic>> getLocalPasses() async {
    return await _methodChannel.invokeListMethod('getLocalPasses');
  }

  static Future<List<dynamic>> getRemotePasses() async {
    return await _methodChannel.invokeListMethod('getRemotePasses');
  }

  static Future<bool> canAddCardWithId(String cardId) async {
    return await _methodChannel
        .invokeMethod<bool>('canAddCardWithId', [cardId]);
  }

  static Future<bool> cardAlreadyAddedToLocalWallet(String cardId) async {
    return await _methodChannel
        .invokeMethod<bool>('cardAlreadyAddedToLocalWallet', [cardId]);
  }

  static Future<bool> cardAlreadyAddedToRemoteWallet(String cardId) async {
    return await _methodChannel
        .invokeMethod<bool>('cardAlreadyAddedToRemoteWallet', [cardId]);
  }

  static void addCard(String userId, String cardId, String cardHolderName,
      String localizedDescription, String cardPanSuffix,
      [String pan, String expiryDate]) {
    _eventChannel.receiveBroadcastStream().listen(_onEvent, onError: _onError);

    if (pan != null && expiryDate != null) {
      _methodChannel.invokeMethod(
          'addCardForUserIdCardIdCardHolderNameCardPanSuffixLocalizedDescriptionPanExpiryDate',
          [
            userId,
            cardId,
            cardHolderName,
            cardPanSuffix,
            localizedDescription,
            pan,
            expiryDate
          ]);
    } else {
      _methodChannel.invokeMethod(
          'addCardForUserIdCardIdCardHolderNameCardPanSuffixLocalizedDescription',
          [
            userId,
            cardId,
            cardHolderName,
            cardPanSuffix,
            localizedDescription
          ]);
    }
  }

  //Event Channel
  static void _onEvent(Object event) {
    FooApplePay.applePayCallback(event, '0', null);
  }

  static void _onError(Object error) {
    FooApplePay.applePayCallback(null, '$error', null);
  }
}
